"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.injectableProps = exports.resolveProps = exports.getProps = exports.getValues = void 0;
const utils_1 = require("./utils");
/**
 * Returns an array of resolved values by the specified token.
 * If a token is not found, then `undefined` value is used.
 *
 * @deprecated Use `tryResolveValues()` function.
 */
function getValues(container, ...tokens) {
    return (0, utils_1.tryResolveValues)(container, ...tokens);
}
exports.getValues = getValues;
/**
 * Returns an object with resolved properties which are specified by token properties.
 * If a token is not found, then `undefined` value is used.
 *
 * @deprecated Use `tryResolveValue()` function.
 *
 * @example
 * ```ts
 * const props = getProps(container, {a: tokenA, b: tokenB});
 * console.log(props); // {a: 1, b: 2}
 * ```
 */
function getProps(container, tokens) {
    return (0, utils_1.tryResolveValue)(container, tokens);
}
exports.getProps = getProps;
/**
 * Returns an object with resolved properties which are specified by token properties.
 * If a token is not found, then `ResolverError` is thrown.
 *
 * @deprecated Use `resolveValue()` function.
 *
 * @example
 * ```ts
 * const props = resolveProps(container, {a: tokenA, b: tokenB});
 * console.log(props); // {a: 1, b: 2}
 * ```
 */
function resolveProps(container, tokens) {
    return (0, utils_1.resolveValue)(container, tokens);
}
exports.resolveProps = resolveProps;
/**
 * Decorates a factory by passing a resolved object with tokens as the first argument.
 * @param factory - A factory.
 * @param tokens - Object with tokens.
 * @return Decorated factory which takes a dependency container as a single argument.
 *
 * @deprecated Use `injectable()` function.
 *
 * @example
 * ```ts
 * const factory = ({a, b}: {a: number, b: number}) => a + b;
 * const decoratedFactory = injectableProps(factory, {a: tokenA, b: tokenB});
 * const result = decoratedFactory(container);
 * ```
 */
function injectableProps(factory, tokens) {
    return (0, utils_1.injectable)(factory, tokens);
}
exports.injectableProps = injectableProps;
//# sourceMappingURL=utils-deprecated.js.map