import { injectable, resolveValue, tryResolveValue, tryResolveValues, } from './utils';
/**
 * Returns an array of resolved values by the specified token.
 * If a token is not found, then `undefined` value is used.
 *
 * @deprecated Use `tryResolveValues()` function.
 */
export function getValues(container, ...tokens) {
    return tryResolveValues(container, ...tokens);
}
/**
 * Returns an object with resolved properties which are specified by token properties.
 * If a token is not found, then `undefined` value is used.
 *
 * @deprecated Use `tryResolveValue()` function.
 *
 * @example
 * ```ts
 * const props = getProps(container, {a: tokenA, b: tokenB});
 * console.log(props); // {a: 1, b: 2}
 * ```
 */
export function getProps(container, tokens) {
    return tryResolveValue(container, tokens);
}
/**
 * Returns an object with resolved properties which are specified by token properties.
 * If a token is not found, then `ResolverError` is thrown.
 *
 * @deprecated Use `resolveValue()` function.
 *
 * @example
 * ```ts
 * const props = resolveProps(container, {a: tokenA, b: tokenB});
 * console.log(props); // {a: 1, b: 2}
 * ```
 */
export function resolveProps(container, tokens) {
    return resolveValue(container, tokens);
}
/**
 * Decorates a factory by passing a resolved object with tokens as the first argument.
 * @param factory - A factory.
 * @param tokens - Object with tokens.
 * @return Decorated factory which takes a dependency container as a single argument.
 *
 * @deprecated Use `injectable()` function.
 *
 * @example
 * ```ts
 * const factory = ({a, b}: {a: number, b: number}) => a + b;
 * const decoratedFactory = injectableProps(factory, {a: tokenA, b: tokenB});
 * const result = decoratedFactory(container);
 * ```
 */
export function injectableProps(factory, tokens) {
    return injectable(factory, tokens);
}
//# sourceMappingURL=utils-deprecated.js.map